@@ -2,6 +2,7 @@
 This module provides an interface for invoking CMake executable.
 """
 
+from __future__ import print_function
 import argparse
 import distutils.sysconfig as du_sysconfig
 import glob
@@ -167,12 +168,13 @@ class CMaker(object):
         if cli_generator_name is not None:
             generator_name = cli_generator_name
 
-        # use the generator returned from the platform, with the current
-        # generator_name as a suggestion
+        # if arch is provided on command line, use it
+        clargs, cli_arch = pop_arg('-A', clargs)
+
         generator = self.platform.get_best_generator(
             generator_name, skip_generator_test=skip_generator_test,
             cmake_executable=self.cmake_executable, cmake_args=clargs,
-            languages=languages, cleanup=cleanup)
+            languages=languages, cleanup=cleanup, architecture=cli_arch)
 
         if not os.path.exists(CMAKE_BUILD_DIR()):
             os.makedirs(CMAKE_BUILD_DIR())
@@ -201,12 +203,17 @@ class CMaker(object):
                 python_include_dir),
             ("-DPYTHON_LIBRARY:FILEPATH=" +
                 python_library),
-            ("-DSKBUILD:BOOL=" +
+            ("-DSKBUILD:INTERNAL=" +
                 "TRUE"),
             ("-DCMAKE_MODULE_PATH:PATH=" +
                 os.path.join(os.path.dirname(__file__), "resources", "cmake"))
         ]
 
+        if generator.toolset:
+            cmd.extend(['-T', generator.toolset])
+        if generator.architecture:
+            cmd.extend(['-A', generator.architecture])
+
         cmd.extend(clargs)
 
         cmd.extend(
@@ -216,6 +223,15 @@ class CMaker(object):
 
         # changes dir to cmake_build and calls cmake's configure step
         # to generate makefile
+        print(
+            "Configuring Project\n"
+            "  Working directory:\n"
+            "    {}\n"
+            "  Command:\n"
+            "    {}\n".format(
+                os.path.abspath(CMAKE_BUILD_DIR()),
+                self._formatArgsForDisplay(cmd)
+            ))
         rtn = subprocess.call(cmd, cwd=CMAKE_BUILD_DIR(), env=generator.env)
         if rtn != 0:
             raise SKBuildError(
@@ -260,7 +276,7 @@ class CMaker(object):
         # if Python.h not found (or python_include_dir is None), try to find a
         # suitable include dir
         found_python_h = (
-            python_include_dir is not None or
+            python_include_dir is not None and
             os.path.exists(os.path.join(python_include_dir, 'Python.h'))
         )
 
@@ -314,6 +330,15 @@ class CMaker(object):
             if python_version:
                 candidate_versions += ('',)
 
+                pymalloc = None
+                try:
+                    pymalloc = bool(sysconfig.get_config_var('WITH_PYMALLOC'))
+                except AttributeError:
+                    pass
+
+                if pymalloc:
+                    candidate_versions += (python_version + 'm',)
+
             candidates = (
                 os.path.join(prefix, ''.join(('python', ver)))
                 for (prefix, ver) in itertools.product(
@@ -341,11 +366,15 @@ class CMaker(object):
         python_library = sysconfig.get_config_var('LIBRARY')
 
         # if static (or nonexistent), try to find a suitable dynamic libpython
-        if (python_library is None or
+        if (not python_library or
                 os.path.splitext(python_library)[1][-2:] == '.a'):
 
             candidate_lib_prefixes = ['', 'lib']
 
+            candidate_implementations = ['python']
+            if hasattr(sys, "pypy_version_info"):
+                candidate_implementations = ['pypy-c', 'pypy3-c']
+
             candidate_extensions = ['.lib', '.so', '.a']
             if sysconfig.get_config_var('WITH_DYLD'):
                 candidate_extensions.insert(0, '.dylib')
@@ -381,10 +410,11 @@ class CMaker(object):
             candidates = (
                 os.path.join(
                     libdir,
-                    ''.join((pre, 'python', ver, abi, ext))
+                    ''.join((pre, impl, ver, abi, ext))
                 )
-                for (pre, ext, ver, abi) in itertools.product(
+                for (pre, impl, ext, ver, abi) in itertools.product(
                     candidate_lib_prefixes,
+                    candidate_implementations,
                     candidate_extensions,
                     candidate_versions,
                     candidate_abiflags
@@ -422,7 +452,10 @@ class CMaker(object):
                 if os.path.splitext(filename)[1] != ".cmake":
                     continue
 
-                for line in io.open(os.path.join(root, filename), encoding="utf-8"):
+                with io.open(os.path.join(root, filename), encoding="utf-8") as fp:
+                    lines = fp.readlines()
+
+                for line in lines:
                     match = RE_FILE_INSTALL.match(line)
                     if match is None:
                         continue
@@ -511,4 +544,9 @@ class CMaker(object):
         Currently, the only formatting is naively surrounding each argument with
         quotation marks.
         """
-        return ' '.join("\"{}\"".format(arg) for arg in args)
+        try:
+            from shlex import quote
+        except ImportError:
+            from pipes import quote
+
+        return ' '.join(quote(arg) for arg in args)
